#!/usr/bin/env python

import argparse
import sys
from helpers import * 

def main(): 

  parser = argparse.ArgumentParser()
  parser.add_argument("repo_name", help="githubuser/repo")
  parser.add_argument("-g", "--generate-html", action='store_true', help="Generate output as HTML")
  parser.add_argument("-l", "--limit", help="Limit to the last x Issues/Pull Requests", type=int, default=0)
  args = parser.parse_args()  

  repo_name = args.repo_name

  if not repo_exists(repo_name): 
    print "Repo does not exist: %s" % repo_name
    sys.exit(1)

  code_contributors = get_code_contributors(repo_name)
  code_commentors = get_code_commentors(repo_name, args.limit)

  non_code_contributors = []
  for user in code_commentors:
    user_name, avatar = user 
    if user_name not in code_contributors:
      non_code_contributors.append({'user_name': user_name, 'avatar': avatar})

  print "Code contributions: %d" % len(code_contributors)
  print "Non-code contributions: %d" % len(non_code_contributors)

  if args.generate_html:
    html_file = "%s_contrib.html" % repo_name.replace("/", "_")
    f = open(html_file, "w")

    title = "Non-code contributions for %s" % repo_name
    f.write("<title>%s</title>\n" %  title)
    f.write("<h1>%s</h1>\n" % title)
    if args.limit > 0:
      f.write("(for the most recent %d issues/pull requests)" % args.limit)
    for user in non_code_contributors: 
      url = "https://github.com/%s/issues?q=involves:%s" % (repo_name, user["user_name"])
      f.write("<a href='%s'><img src='%s' width='128'></a>\n" % (url, user["avatar"]))
    f.write("<br><br>Generated by <a href='https://github.com/glasnt/octohat'>octohat</a>")

    print "Generated HTML representation, saved to %s" % html_file
  else: 
    for user in non_code_contributors: 
      print user["user_name"]

if __name__ == "__main__":
    main()



# Copyright (c) 2013 Alon Swartz <alon@turnkeylinux.org>
#
# This file is part of OctoHub.
#
# OctoHub is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.

import simplejson as json

class ResponseError(Exception):
    """Accessible attributes: error
        error (AttrDict): Parsed error response
    """
    def __init__(self, error):
        Exception.__init__(self, error)
        self.error = error

    def __str__(self):
        return json.dumps(self.error, indent=1)


class OctoHubError(Exception):
    pass




# Copyright (c) 2013 Alon Swartz <alon@turnkeylinux.org>
#
# This file is part of OctoHub.
#
# OctoHub is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.

import requests

from response import parse_response

class Pager(object):
    def __init__(self, conn, uri, params, max_pages=0):
        """Iterator object handling pagination of Connection.send (method: GET)
            conn (octohub.Connection): Connection object
            uri (str): Request URI (e.g., /user/issues)
            params (dict): Parameters to include in request
            max_pages (int): Maximum amount of pages to get (0 for all)
        """
        self.conn = conn
        self.uri = uri
        self.params = params
        self.max_pages = max_pages
        self.count = 0

    def __iter__(self):
        while True:
            self.count += 1
            response = self.conn.send('GET', self.uri, self.params)
            yield response

            if self.count == self.max_pages:
                break

            if not 'next' in response.parsed_link.keys():
                break

            self.uri = response.parsed_link.next.uri
            self.params = response.parsed_link.next.params

class Connection(object):
    def __init__(self, token=None):
        """OctoHub connection
            token (str): GitHub Token (anonymous if not provided)
        """
        self.endpoint = 'https://api.github.com'
        self.headers = {'User-Agent': 'octohub'}

        if token:
            self.headers['Authorization'] = 'token %s' % token

    def send(self, method, uri, params={}, data=None):
        """Prepare and send request
            method (str): Request HTTP method (e.g., GET, POST, DELETE, ...)
            uri (str): Request URI (e.g., /user/issues)
            params (dict): Parameters to include in request
            data (str | file type object): data to include in request

            returns: requests.Response object, including:
                response.parsed (AttrDict): parsed response when applicable
                response.parsed_link (AttrDict): parsed header link when applicable
                http://docs.python-requests.org/en/latest/api/#requests.Response
        """
        url = self.endpoint + uri
        kwargs = {'headers': self.headers, 'params': params, 'data': data}
        response = requests.request(method, url, **kwargs)

        return parse_response(response)

